import Observable from "zen-observable";
import { Thread } from "./thread";
export { Thread };
export declare namespace Pool {
    type Event<ThreadType extends Thread = any> = PoolEvent<ThreadType>;
    type EventType = PoolEventType;
}
export declare enum PoolEventType {
    initialized = "initialized",
    taskCompleted = "taskCompleted",
    taskFailed = "taskFailed",
    taskQueued = "taskQueued",
    taskStart = "taskStart",
    terminated = "terminated"
}
export interface Task<ThreadType extends Thread, Return> {
    id: number;
    run: TaskRunFunction<ThreadType, Return>;
}
declare type TaskRunFunction<ThreadType extends Thread, Return> = (worker: ThreadType) => Promise<Return>;
export declare type PoolEvent<ThreadType extends Thread> = {
    type: PoolEventType.initialized;
    size: number;
} | {
    type: PoolEventType.taskQueued;
    taskID: number;
} | {
    type: PoolEventType.taskStart;
    taskID: number;
    workerID: number;
} | {
    type: PoolEventType.taskCompleted;
    returnValue: any;
    taskID: number;
    workerID: number;
} | {
    type: PoolEventType.taskFailed;
    error: Error;
    taskID: number;
    workerID: number;
} | {
    type: PoolEventType.terminated;
    remainingQueue: Array<Task<ThreadType, any>>;
};
export interface Pool<ThreadType extends Thread> {
    events(): Observable<PoolEvent<ThreadType>>;
    queue<Return>(task: TaskRunFunction<ThreadType, Return>): Promise<Return>;
    terminate(force?: boolean): Promise<void>;
}
export interface PoolOptions {
    concurrency?: number;
    name?: string;
    size?: number;
}
declare function PoolConstructor<ThreadType extends Thread>(spawnWorker: () => Promise<ThreadType>, optionsOrSize?: number | PoolOptions): Pool<ThreadType>;
export declare const Pool: typeof PoolConstructor & {
    EventType: typeof PoolEventType;
};
